@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 from functools import partial
 from operator import getitem
 
@@ -5,8 +7,9 @@ import numpy as np
 
 from dask import core
 from dask.array.core import Array, apply_infer_dtype, asarray, blockwise, elemwise
-from dask.base import is_dask_collection, normalize_function
+from dask.base import is_dask_collection
 from dask.highlevelgraph import HighLevelGraph
+from dask.tokenize import normalize_token
 from dask.utils import derived_from, funcname
 
 
@@ -40,7 +43,7 @@ class da_frompyfunc:
         return "da.frompyfunc<%s, %d, %d>" % (self._name, self.nin, self.nout)
 
     def __dask_tokenize__(self):
-        return (normalize_function(self._func), self.nin, self.nout)
+        return (normalize_token(self._func), self.nin, self.nout)
 
     def __reduce__(self):
         return (da_frompyfunc, (self._func, self.nin, self.nout))
@@ -89,6 +92,9 @@ class ufunc:
         if isinstance(ufunc, np.ufunc):
             derived_from(np)(self)
 
+    def __dask_tokenize__(self):
+        return self.__name__, normalize_token(self._ufunc)
+
     def __getattr__(self, key):
         if key in self._forward_attrs:
             return getattr(self._ufunc, key)
