@@ -187,6 +187,12 @@ def start_state_from_dask(dsk, cache=None, sortkey=None, keys=None):
         dependencies[key]
         task = dsk.get(key, None)
         if task is None:
+            if dependents[key] and not cache.get(key, None):
+                raise ValueError(
+                    "Missing dependency {} for dependents {}".format(
+                        key, dependents[key]
+                    )
+                )
             continue
         elif isinstance(task, DataNode):
             cache[key] = task()
