@@ -46,12 +46,12 @@ graph types laid out very carefully to show the kinds of situations that often
 arise, and the order we would like to be determined.
 
 """
-import copy
 from collections import defaultdict, deque, namedtuple
 from collections.abc import Callable, Iterable, Mapping, MutableMapping
 from typing import Any, Literal, NamedTuple, overload
 
-from dask.core import get_dependencies, get_deps, getcycle, istask, reverse_dict
+from dask._task_spec import DataNode, DependenciesMapping
+from dask.core import get_deps, getcycle, istask, reverse_dict
 from dask.typing import Key
 
 
@@ -66,8 +66,7 @@ def order(
     dependencies: Mapping[Key, set[Key]] | None = None,
     *,
     return_stats: Literal[True],
-) -> dict[Key, Order]:
-    ...
+) -> dict[Key, Order]: ...
 
 
 @overload
@@ -76,8 +75,7 @@ def order(
     dependencies: Mapping[Key, set[Key]] | None = None,
     *,
     return_stats: Literal[False] = False,
-) -> dict[Key, int]:
-    ...
+) -> dict[Key, int]: ...
 
 
 def order(
@@ -104,23 +102,24 @@ def order(
     {'a': 0, 'c': 1, 'b': 2, 'd': 3}
     """
     if not dsk:
-        return {}  # type: ignore
+        return {}
 
     dsk = dict(dsk)
-    expected_len = len(dsk)
-
-    if dependencies is None:
-        dependencies_are_copy = True
-        dependencies = {k: get_dependencies(dsk, k) for k in dsk}
-    else:
-        # Below we're removing items from the sets in this dict
-        # We need a deepcopy for that but we only want to do this if necessary
-        # since this only happens for special cases.
-        dependencies_are_copy = False
-        dependencies = dict(dependencies)
 
+    dependencies = DependenciesMapping(dsk)
     dependents = reverse_dict(dependencies)
 
+    external_keys = set()
+    if len(dependents) != len(dependencies):
+        # There are external keys. Let's add a DataNode to ensure the algo below
+        # is encountering a complete graph. Those artificial data nodes will be
+        # ignored when assigning results
+        for k in dependents:
+            if k not in dependencies:
+                external_keys.add(k)
+                dsk[k] = DataNode(k, object())
+
+    expected_len = len(dsk)
     leaf_nodes = {k for k, v in dependents.items() if not v}
     root_nodes = {k for k, v in dependencies.items() if not v}
 
@@ -138,6 +137,7 @@ def order(
     all_tasks = False
     n_removed_leaves = 0
     requires_data_task = defaultdict(set)
+
     while not all_tasks:
         all_tasks = True
         for leaf in list(leaf_nodes):
@@ -158,29 +158,27 @@ def order(
                     result[leaf] = prio
                 n_removed_leaves += 1
                 leaf_nodes.remove(leaf)
-                del dsk[leaf]
-                del dependents[leaf]
                 for dep in dependencies[leaf]:
                     dependents[dep].remove(leaf)
                     if not dependents[dep]:
                         leaf_nodes.add(dep)
+                del dsk[leaf]
+                del dependencies[leaf]
+                del dependents[leaf]
 
         for root in list(root_nodes):
             if root in leaf_nodes:
                 continue
-            if not istask(dsk[root]) and len(dependents[root]) > 1:
-                if not dependencies_are_copy:
-                    dependencies_are_copy = True
-                    dependencies = copy.deepcopy(dependencies)
-                root_nodes.remove(root)
-                for dep in dependents[root]:
-                    requires_data_task[dep].add(root)
-                    dependencies[dep].remove(root)
-                    if not dependencies[dep]:
-                        root_nodes.add(dep)
+            deps_root = dependents[root]
+            if not istask(dsk[root]) and len(deps_root) > 1:
                 del dsk[root]
                 del dependencies[root]
+                root_nodes.remove(root)
                 del dependents[root]
+                for dep in deps_root:
+                    requires_data_task[dep].add(root)
+                    if not dependencies[dep]:
+                        root_nodes.add(dep)
 
     num_needed, total_dependencies = ndependencies(dependencies, dependents)
     if len(total_dependencies) != len(dsk):
@@ -197,7 +195,7 @@ def order(
     runnable_hull = set()
     reachable_hull = set()
 
-    runnable = []
+    runnable: list[Key] = []
 
     known_runnable_paths: dict[Key, list[list[Key]]] = {}
     known_runnable_paths_pop = known_runnable_paths.pop
@@ -205,12 +203,8 @@ def order(
     crit_path_counter = 0
     scrit_path: set[Key] = set()
     _crit_path_counter_offset: int | float = 0
-
     _sort_keys_cache: dict[Key, tuple[int, int, int, int, str]] = {}
 
-    leafs_connected_to_loaded_roots: set[Key] = set()
-    processed_roots = set()
-
     def sort_key(x: Key) -> tuple[int, int, int, int, str]:
         try:
             return _sort_keys_cache[x]
@@ -229,11 +223,11 @@ def order(
             return rv
 
     def add_to_result(item: Key) -> None:
-        nonlocal crit_path_counter
         # Earlier versions recursed into this method but this could cause
         # recursion depth errors. This is the only reason for the while loop
         next_items = [item]
         nonlocal i
+        nonlocal min_leaf_degree  # type: ignore[misc]
         while next_items:
             item = next_items.pop()
             runnable_hull.discard(item)
@@ -244,16 +238,32 @@ def order(
 
             while requires_data_task[item]:
                 add_to_result(requires_data_task[item].pop())
-
             if return_stats:
                 result[item] = Order(i, crit_path_counter - _crit_path_counter_offset)
             else:
                 result[item] = i
-
+            if item not in external_keys:
+                i += 1
             if item in root_nodes:
-                processed_roots.add(item)
+                for leaf in leafs_connected[item]:
+                    if leaf in leaf_nodes:
+                        degree = leafs_degree.pop(leaf)
+                        leafs_per_degree[degree].remove(leaf)
+
+                        new_degree = degree - 1
+                        if new_degree > 0:
+                            if new_degree < min_leaf_degree:
+                                min_leaf_degree = new_degree
+                            leafs_per_degree[new_degree].add(leaf)
+                            leafs_degree[leaf] = new_degree
+                        elif not leafs_per_degree[degree]:
+                            assert degree == min_leaf_degree
+                            while min_leaf_degree != max_leaf_degree and (
+                                min_leaf_degree not in leafs_per_degree
+                                or not leafs_per_degree[min_leaf_degree]
+                            ):
+                                min_leaf_degree += 1
 
-            i += 1
             # Note: This is a `set` and therefore this introduces a certain
             # randomness. However, this randomness should not have any impact on
             # the final result since the `process_runnable` should produce
@@ -273,7 +283,6 @@ def order(
         # is purely cosmetical and used for some visualizations and I haven't
         # settled on how to implement this best so I didn't want to have large
         # indentations that make things harder to read
-        nonlocal _crit_path_counter_offset
 
         def wrapper(*args: Any, **kwargs: Any) -> None:
             nonlocal _crit_path_counter_offset
@@ -318,16 +327,17 @@ def order(
                     add_to_result(key)
                     continue
                 path = [key]
-                branches = deque([path])
+                branches = deque([(0, path)])
+
                 while branches:
-                    path = branches.popleft()
+                    nsplits, path = branches.popleft()
                     while True:
                         # Loop invariant. Too expensive to compute at runtime
                         # assert not set(known_runnable_paths).intersection(runnable_hull)
                         current = path[-1]
                         runnable_hull.add(current)
                         deps_downstream = dependents[current]
-                        deps_upstream = dependencies[current]  # type: ignore
+                        deps_upstream = dependencies[current]
                         if not deps_downstream:
                             # FIXME: The fact that it is possible for
                             # num_needed[current] == 0 means we're doing some
@@ -339,14 +349,15 @@ def order(
                                 runnable_hull.discard(current)
                         elif len(path) == 1 or len(deps_upstream) == 1:
                             if len(deps_downstream) > 1:
+                                nsplits += 1
                                 for d in sorted(deps_downstream, key=sort_key):
                                     # This ensures we're only considering splitters
                                     # that are genuinely splitting and not
                                     # interleaving
-                                    if len(dependencies[d]) == 1:  # type: ignore
+                                    if len(dependencies[d]) == 1:
                                         branch = path.copy()
                                         branch.append(d)
-                                        branches.append(branch)
+                                        branches.append((nsplits, branch))
                                 break
                             path.extend(deps_downstream)
                             continue
@@ -366,6 +377,13 @@ def order(
                                         pruned_branches
                                     )
                                 else:
+                                    if nsplits > 1:
+                                        path = []
+                                        for pruned in pruned_branches:
+                                            path.extend(pruned)
+                                        branches.append((nsplits - 1, path))
+                                        break
+
                                     while pruned_branches:
                                         path = pruned_branches.popleft()
                                         for k in path:
@@ -375,7 +393,7 @@ def order(
                                             add_to_result(k)
                         else:
                             if (
-                                len(dependencies[current]) > 1  # type: ignore
+                                len(dependencies[current]) > 1
                                 and num_needed[current] <= 1
                             ):
                                 for k in path:
@@ -393,42 +411,38 @@ def order(
     # writing, the most expensive part of ordering is the prep work (mostly
     # connected roots + sort_key) which can be reused for multiple orderings.
 
-    def get_target(longest_path: bool = False) -> Key:
-        # Some topologies benefit if the node with the most dependencies
-        # is used as first choice, others benefit from the opposite.
-        candidates = leaf_nodes
-        skey: Callable = sort_key
-        if reachable_hull:
-            skey = lambda k: (num_needed[k], sort_key(k))
+    # Degree in this context is the number root nodes that have to be loaded for
+    # this leaf to become accessible. Zero means the leaf is already accessible
+    # in which case it _should_ either already be in result or be accessible vie
+    # process_runnables
+    # When picking a new target, we prefer the leafs with the least number of
+    # roots that need loading.
+    leafs_degree = {}
+    leafs_per_degree = defaultdict(set)
+    min_leaf_degree = len(dsk)
+    max_leaf_degree = len(dsk)
+    for leaf in leaf_nodes - root_nodes:
+        degree = len(roots_connected[leaf])
+        min_leaf_degree = min(min_leaf_degree, degree)
+        max_leaf_degree = max(max_leaf_degree, degree)
+        leafs_degree[leaf] = degree
+        leafs_per_degree[degree].add(leaf)
 
-        all_leafs_accessible = len(leafs_connected_to_loaded_roots) >= len(candidates)
-        if reachable_hull and not all_leafs_accessible:
-            # Avoid this branch if we can already access all leafs. Just pick
-            # one of them without the expensive selection process in this case.
-
-            candidates = reachable_hull & candidates
+    def get_target() -> Key:
+        # If we're already mid run and there is a runnable_hull we'll attempt to
+        # pick the next target in a way that minimizes the number of additional
+        # root nodes that are needed
+        all_leafs_accessible = min_leaf_degree == max_leaf_degree
+        is_trivial_lookup = not reachable_hull or all_leafs_accessible
+        if not is_trivial_lookup:
+            candidates = reachable_hull & leafs_per_degree[min_leaf_degree]
             if not candidates:
-                # We can't reach a leaf node directly, but we still have nodes
-                # with results in memory, these notes can inform our path towards
-                # a new preferred leaf node.
-                for r in processed_roots:
-                    leafs_connected_to_loaded_roots.update(leafs_connected[r])
-                processed_roots.clear()
-
-                leafs_connected_to_loaded_roots.intersection_update(leaf_nodes)
-                candidates = leafs_connected_to_loaded_roots
-            else:
-                leafs_connected_to_loaded_roots.update(candidates)
-
-        elif not reachable_hull:
-            # We reach a new and independent branch, so throw away previous branch
-            leafs_connected_to_loaded_roots.clear()
-
-        if longest_path and (not reachable_hull or all_leafs_accessible):
-            return leaf_nodes_sorted.pop()
+                candidates = leafs_per_degree[min_leaf_degree]
+            # Even without reachable hull overlap this should be relatively
+            # small so one full pass should be fine
+            return min(candidates, key=sort_key)
         else:
-            # FIXME: This can be very expensive
-            return min(candidates, key=skey)
+            return leaf_nodes_sorted.pop()
 
     def use_longest_path() -> bool:
         size = 0
@@ -447,10 +461,10 @@ def order(
 
         return True
 
+    # Some topologies benefit if the node with the most dependencies
+    # is used as first choice, others benefit from the opposite.
     longest_path = use_longest_path()
-    leaf_nodes_sorted = []
-    if longest_path:
-        leaf_nodes_sorted = sorted(leaf_nodes, key=sort_key, reverse=False)
+    leaf_nodes_sorted = sorted(leaf_nodes, key=sort_key, reverse=not longest_path)
 
     # *************************************************************************
     # CORE ALGORITHM STARTS HERE
@@ -550,14 +564,14 @@ def order(
         assert not scrit_path
 
         # A. Build the critical path
-        target = get_target(longest_path=longest_path)
+        target = get_target()
         next_deps = dependencies[target]
         path_append(target)
 
         while next_deps:
             item = max(next_deps, key=sort_key)
             path_append(item)
-            next_deps = dependencies[item]
+            next_deps = dependencies[item].difference(result)
             path_extend(next_deps)
 
         # B. Walk the critical path
@@ -597,12 +611,14 @@ def order(
         process_runnables()
 
     assert len(result) == expected_len
+    for k in external_keys:
+        del result[k]
     return result  # type: ignore
 
 
 def _connecting_to_roots(
     dependencies: Mapping[Key, set[Key]], dependents: Mapping[Key, set[Key]]
-) -> tuple[dict[Key, set[Key]], dict[Key, int]]:
+) -> tuple[dict[Key, frozenset[Key]], dict[Key, int]]:
     """Determine for every node which root nodes are connected to it (i.e.
     ancestors). If arguments of dependencies and dependents are switched, this
     can also be used to determine which leaf nodes are connected to which node
@@ -627,47 +643,61 @@ def _connecting_to_roots(
             # benefit from the speedup of using integers would be to convert
             # this back on demand which makes the code very hard to read.
             roots.add(k)
-            result[k] = {k}
+            result[k] = frozenset({k})
             deps = dependents[k]
             max_dependents[k] = len(deps)
             for child in deps:
                 num_needed[child] -= 1
                 if not num_needed[child]:
                     current.append(child)
+
+    dedup_mapping: dict[frozenset[Key], frozenset[Key]] = {}
     while current:
         key = current.pop()
+        if key in result:
+            continue
         for parent in dependents[key]:
             num_needed[parent] -= 1
             if not num_needed[parent]:
                 current.append(parent)
         # At some point, all the roots are the same, particularly for dense
         # graphs. We don't want to create new sets over and over again
-        new_set = None
-        identical_sets = True
-        result_first = None
-
+        transitive_deps = []
+        transitive_deps_ids = set()
+        max_dependents_key = list()
         for child in dependencies[key]:
             r_child = result[child]
-            if not result_first:
-                result_first = r_child
-                max_dependents[key] = max_dependents[child]
-            # This clause is written such that it can circuit break early
-            elif not (  # type: ignore[unreachable]
-                identical_sets
-                and (result_first is r_child or r_child.issubset(result_first))
-            ):
-                identical_sets = False
-                if not new_set:
-                    new_set = result_first.copy()
-                max_dependents[key] = max(max_dependents[child], max_dependents[key])
-                new_set.update(r_child)
+            if id(r_child) in transitive_deps_ids:
+                continue
+            transitive_deps.append(r_child)
+            transitive_deps_ids.add(id(r_child))
+            max_dependents_key.append(max_dependents[child])
 
-        assert new_set is not None or result_first is not None
-        result[key] = new_set or result_first
+        max_dependents[key] = max(max_dependents_key)
+        if len(transitive_deps_ids) == 1:
+            result[key] = transitive_deps[0]
+        else:
+            d = transitive_deps[0]
+            if all(tdeps.issubset(d) for tdeps in transitive_deps[1:]):
+                result[key] = d
+            else:
+                res = set(d)
+                for tdeps in transitive_deps[1:]:
+                    res.update(tdeps)
+                # frozenset is unfortunately triggering a copy. In the event of
+                # a cache hit, this is wasted time but we can't hash the set
+                # otherwise (unless we did it manually) and can therefore not
+                # deduplicate without this copy
+                frozen_res = frozenset(res)
+                del res, tdeps
+                try:
+                    result[key] = dedup_mapping[frozen_res]
+                except KeyError:
+                    dedup_mapping[frozen_res] = frozen_res
+                    result[key] = frozen_res
+    del dedup_mapping
 
-    # The order algo doesn't care about this but this makes it easier to
-    # understand and shouldn't take that much time
-    empty_set: set[Key] = set()
+    empty_set: frozenset[Key] = frozenset()
     for r in roots:
         result[r] = empty_set
     return result, max_dependents
@@ -794,40 +824,17 @@ def diagnostics(
     return rv, pressure
 
 
-def _f() -> None:
-    ...
-
-
-def _convert_task(task: Any) -> Any:
-    if istask(task):
-        assert callable(task[0])
-        new_spec: list[Any] = []
-        for el in task[1:]:
-            if isinstance(el, (str, int)):
-                new_spec.append(el)
-            elif isinstance(el, tuple):
-                if istask(el):
-                    new_spec.append(_convert_task(el))
-                else:
-                    new_spec.append(el)
-            elif isinstance(el, list):
-                new_spec.append([_convert_task(e) for e in el])
-        return (_f, *new_spec)
-    elif isinstance(task, tuple):
-        return (_f, task)
-    else:
-        return (_f, *task)
+def _f() -> None: ...
 
 
 def sanitize_dsk(dsk: MutableMapping[Key, Any]) -> dict:
     """Take a dask graph and replace callables with a dummy function and remove
     payload data like numpy arrays, dataframes, etc.
     """
+    from dask._task_spec import Task, TaskRef
+
     new = {}
-    for key, values in dsk.items():
-        new_key = key
-        new[new_key] = _convert_task(values)
-    if get_deps(new) != get_deps(dsk):
-        # The switch statement in _convert likely dropped some keys
-        raise RuntimeError("Sanitization failed to preserve topology.")
+    deps = DependenciesMapping(dsk)
+    for key, values in deps.items():
+        new[key] = Task(key, _f, *(TaskRef(k) for k in values))
     return new
