@@ -16,6 +16,8 @@ from tlz import merge
 import dask
 import dask.bag as db
 from dask import compute
+from dask._task_spec import Task
+from dask.base import collections_to_expr
 from dask.delayed import Delayed, delayed, to_task_dask
 from dask.highlevelgraph import HighLevelGraph
 from dask.utils_test import inc
@@ -23,6 +25,7 @@ from dask.utils_test import inc
 
 class Tuple:
     __dask_scheduler__ = staticmethod(dask.threaded.get)
+    __dask_optimize__ = None
 
     def __init__(self, dsk, keys):
         self._dask = dsk
@@ -79,7 +82,8 @@ def test_to_task_dask():
     task, dask = to_task_dask(x)
     assert task in dask
     f = dask.pop(task)
-    assert f == (tuple, ["a", "b", "c"])
+    assert f.func == tuple
+    assert f.dependencies == {"a", "b", "c"}
     assert dask == x._dask
 
 
@@ -91,7 +95,6 @@ def test_delayed():
 
     a = delayed(1)
     assert a.compute() == 1
-    assert 1 in a.dask.values()
     b = add2(add2(a, 2), 3)
     assert a.key in b.dask
 
@@ -102,6 +105,7 @@ def test_delayed_with_namedtuple():
 
     literal = dask.delayed(3)
     with_class = dask.delayed({"a": ANamedTuple(a=literal)})
+    assert with_class.compute() == {"a": ANamedTuple(a=3)}
 
     def return_nested(obj):
         return obj["a"].a
@@ -271,7 +275,9 @@ def test_method_getattr_call_same_task():
     a = delayed([1, 2, 3])
     o = a.index(1)
     # Don't getattr the method, then call in separate task
-    assert getattr not in {v[0] for v in o.__dask_graph__().values()}
+    tasks = {v.func for v in o.__dask_graph__().values() if isinstance(v, Task)}
+    assert tasks
+    assert getattr not in tasks
 
 
 def test_np_dtype_of_delayed():
@@ -327,16 +333,6 @@ def test_common_subexpressions():
     assert len(res.dask) == 3
 
 
-def test_delayed_optimize():
-    x = Delayed("b", {"a": 1, "b": (inc, "a"), "c": (inc, "b")})
-    (x2,) = dask.optimize(x)
-    # Delayed's __dask_optimize__ culls out 'c'
-    assert sorted(x2.dask.keys()) == ["a", "b"]
-    assert x2._layer != x2._key
-    # Optimize generates its own layer name, which doesn't match the key.
-    # `Delayed._rebuild` handles this.
-
-
 def test_lists():
     a = delayed(1)
     b = delayed(2)
@@ -518,7 +514,8 @@ def test_nout():
 def test_nout_with_tasks(x):
     length = len(x)
     d = delayed(x, nout=length)
-    assert len(d) == len(list(d)) == length
+    assert len(d) == length
+    assert len(list(d)) == length
     assert d.compute() == x
 
 
@@ -570,9 +567,67 @@ def test_array_delayed():
     assert (delayed_arr.compute() == arr).all()
 
 
-def test_array_bag_delayed():
-    da = pytest.importorskip("dask.array")
+def test_array_delayed_complex_optimization():
+    # Ensure that when multiple collections are passed to a Delayed function,
+    # they are optimized together
     np = pytest.importorskip("numpy")
+    pytest.importorskip("dask.array")
+    from dask.array.core import from_func
+    from dask.array.utils import assert_eq
+
+    called = False
+
+    def only_once():
+        nonlocal called
+        if called:
+            raise RuntimeError("Already executed")
+
+        called = True
+        return np.arange(100).reshape((10, 10))
+
+    darr = from_func(only_once, shape=(10, 10), dtype=int)
+    a = darr + 1
+    b = darr + 2
+    val = delayed(sum)([a, b, 1])
+    assert isinstance(val, Delayed)
+    np_arr = np.arange(100).reshape((10, 10))
+    assert_eq(val.compute(), (np_arr + 1) + (np_arr + 2) + 1)
+
+
+def test_array_delayed_complex_optimization_kwargs():
+    # Ensure that collections that if multiple collections are passed to a
+    # Delayed function that they are optimized together
+    np = pytest.importorskip("numpy")
+    pytest.importorskip("dask.array")
+    from dask.array.core import from_func
+    from dask.array.utils import assert_eq
+
+    called = False
+
+    def only_once():
+        nonlocal called
+        if called:
+            raise RuntimeError("Already executed")
+
+        called = True
+        return np.arange(100).reshape((10, 10))
+
+    darr = from_func(only_once, shape=(10, 10), dtype=int)
+    a = darr + 1
+    b = darr + 2
+
+    def sum_kwargs_only(*, a, b, c):
+        return sum([a, b, c])
+
+    val = delayed(sum_kwargs_only)(a=a, b=b, c=1)
+    assert isinstance(val, Delayed)
+    np_arr = np.arange(100).reshape((10, 10))
+    assert_eq(val.compute(), (np_arr + 1) + (np_arr + 2) + 1)
+
+
+def test_array_bag_delayed():
+    np = pytest.importorskip("numpy")
+    da = pytest.importorskip("dask.array")
 
     arr1 = np.arange(100).reshape((10, 10))
     arr2 = arr1.dot(arr1.T)
@@ -619,9 +674,7 @@ def test_delayed_method_descriptor():
 def test_delayed_callable():
     f = delayed(add, pure=True)
     v = f(1, 2)
-    assert v.dask == {v.key: (add, 1, 2)}
-
-    assert f.dask == {f.key: add}
+    assert v.compute() == 3
     assert f.compute() == add
 
 
@@ -675,6 +728,7 @@ def test_delayed_name():
 
 
 def test_finalize_name():
+    pytest.importorskip("numpy")
     da = pytest.importorskip("dask.array")
 
     x = da.ones(10, chunks=5)
@@ -691,6 +745,7 @@ def test_finalize_name():
 
 
 def test_keys_from_array():
+    pytest.importorskip("numpy")
     da = pytest.importorskip("dask.array")
     from dask.array.utils import _check_dsk
 
@@ -752,6 +807,7 @@ def test_attribute_of_attribute():
 
 
 def test_check_meta_flag():
+    pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
     from pandas import Series
 
@@ -833,14 +889,10 @@ def test_annotations_survive_optimization():
     assert len(d.dask.layers) == 1
     assert len(d.dask.layers["b"]) == 3
     assert d.dask.layers["b"].annotations == {"foo": "bar"}
-
-    # Ensure optimizing a Delayed object returns a HighLevelGraph
-    # and doesn't loose annotations
-    (d_opt,) = dask.optimize(d)
-    assert type(d_opt.dask) is HighLevelGraph
-    assert len(d_opt.dask.layers) == 1
-    assert len(d_opt.dask.layers["b"]) == 2  # c is culled
-    assert d_opt.dask.layers["b"].annotations == {"foo": "bar"}
+    optimized = collections_to_expr([d]).optimize()
+    assert optimized.__dask_annotations__() == {
+        "foo": {k: "bar" for k in optimized.__dask_graph__()}
+    }
 
 
 def test_delayed_function_attributes_forwarded():
@@ -852,3 +904,60 @@ def test_delayed_function_attributes_forwarded():
     assert add.__name__ == "add"
     assert add.__doc__ == "This is a docstring"
     assert add.__wrapped__(1, 2) == 3
+
+
+def test_delayed_fusion():
+    @delayed
+    def test(i):
+        return i + 1
+
+    @delayed
+    def test2(i):
+        return i + 2
+
+    @delayed
+    def test3(i):
+        return i + 3
+
+    obj = test3(test2(test(10)))
+    dsk = collections_to_expr([obj]).__dask_graph__()
+    assert len(dsk) == 3
+
+    obj2 = test3(test2(test(10)))
+    with dask.config.set({"optimization.fuse.delayed": True}):
+        dsk2 = collections_to_expr([obj]).optimize().__dask_graph__()
+        result = dask.compute(obj2)
+    assert len(dsk2) == 2
+    assert dask.compute(obj) == result
+
+
+def test_p2p_as_delayed():
+    pytest.importorskip("distributed")
+    pytest.importorskip("numpy")
+    pytest.importorskip("dask.dataframe")
+    from distributed.utils_test import gen_cluster
+
+    import dask.dataframe as dd
+
+    @gen_cluster(client=True)
+    async def _test(c, s, *workers):
+        test_df = dd.from_dict(
+            {
+                "partition": [0, 1, 2, 3, 0, 1, 2, 3],
+                "value": [1, 2, 3, 4, 5, 6, 7, 8],
+            },
+            npartitions=2,
+        )
+        part_df = test_df.shuffle("partition", force=True, shuffle_method="p2p")
+
+        @delayed
+        def delayed_func(x):
+            return x
+
+        delay_df = delayed_func(part_df)
+        dd.utils.assert_eq(
+            await c.gather(c.compute(delay_df)),
+            await c.gather(c.compute(part_df)),
+        )
+
+    _test()
