@@ -848,6 +848,7 @@ def test_nested_tokenize_seen():
 
 def test_tokenize_dict():
     # Insertion order is ignored. Keys can be an unsortable mix of types.
+    assert check_tokenize({"x": 1, 1: "x"}) == "ba2498a4c5583cc3ec540865185cdafd"
     assert check_tokenize({"x": 1, 1: "x"}) == check_tokenize({1: "x", "x": 1})
     assert check_tokenize({"x": 1, 1: "x"}) != check_tokenize({"x": 1, 2: "x"})
     assert check_tokenize({"x": 1, 1: "x"}) != check_tokenize({"x": 2, 1: "x"})
@@ -1481,3 +1482,43 @@ def test_tokenize_nested_sequence_thread_safe():
         assert len({f.result() for f in futures}) == 1
         futures = [pool.submit(normalize_token, nested_dict) for _ in range(1000)]
         assert len({f.result() for f in futures}) == 1
+
+
+def test_tokenize_range_index():
+    np = pytest.importorskip("numpy")
+    pd = pytest.importorskip("pandas")
+    a = pd.RangeIndex(0, 10)
+    b = pd.RangeIndex(0, 10)
+    c = pd.RangeIndex(0, 20)
+    assert check_tokenize(a) == check_tokenize(b)
+    assert check_tokenize(a) != check_tokenize(c)
+
+    d = pd.RangeIndex(0, 10, step=2)
+    e = pd.RangeIndex(0, 10, step=3)
+    assert check_tokenize(d) != check_tokenize(e)
+
+    large = pd.RangeIndex(0, 1000000)
+    array_token = normalize_token(np.zeros(10))
+    assert isinstance(array_token[0], str)
+    # hex_buffer_output_len
+    # It's difficult to assert what actually goes into the tokens
+    # Therefore we just check if there is a string of the appropriate length in
+    # the tokens below This check is to verify the assumption and to avoid hard
+    # coding a length. This seems to not be stable accross python versions or
+    # platforms but there is a minimal length
+    hex_hash_len = len(array_token[0])
+    assert hex_hash_len >= 32
+
+    def assert_no_hashes(tokens):
+        if isinstance(tokens, (list, tuple)):
+            for token in tokens:
+                assert_no_hashes(token)
+        elif isinstance(tokens, dict):
+            for key, value in tokens.items():
+                assert_no_hashes(key)
+                assert_no_hashes(value)
+        elif isinstance(tokens, str):
+            if len(tokens) == hex_hash_len:
+                raise AssertionError("found a hash")
+
+    assert_no_hashes(normalize_token(large))
